/*
 * libmfcr2 - a library for MFC/R2 signaling on E1 lines
 *
 * libmfcr2.h
 *
 * Written by Steve Underwood <steveu@coppice.org>
 *
 * Copyright (C) 2002 Steve Underwood
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#if !defined(_LIBMFCR2_H_)
#define _LIBMFCR2_H_

/* Universal call clearing causes, based on the ISDN ones */

#define UC_UNASSIGNED_NUMBER            0x01
#define UC_NO_ROUTE                     0x02
#define UC_CHANNEL_UNACCEPTABLE         0x06
#define UC_NORMAL_CLEARING              0x10
#define UC_USER_BUSY                    0x11
#define UC_NO_USER_RESPONDING           0x12
#define UC_NO_ANSWER_FROM_USER          0x13
#define UC_CALL_REJECTED                0x15
#define UC_NUMBER_CHANGED               0x16
#define UC_DEST_OUT_OF_ORDER            0x1B
#define UC_INVALID_NUMBER_FORMAT        0x1C
#define UC_FACILITY_REJECTED            0x1D
#define UC_RESP_TO_STAT_ENQ             0x1E
#define UC_UNSPECIFIED_CAUSE            0x1F
#define UC_NO_CIRCUIT_AVAILABLE         0x22
#define UC_NETWORK_OUT_OF_ORDER         0x26
#define UC_TEMPORARY_FAILURE            0x29
#define UC_NETWORK_CONGESTION           0x2A
#define UC_ACCESS_INFO_DISCARDED        0x2B
#define UC_REQ_CHANNEL_NOT_AVAILABLE	0x2C
#define UC_PRE_EMPTED                   0x2D
#define UC_FACILITY_NOT_SUBSCRIBED      0x32
#define UC_OUTGOING_CALL_BARRED         0x34
#define UC_INCOMING_CALL_BARRED         0x36
#define UC_BEAR_CAP_NOT_AVAILABLE       0x3A
#define UC_SERVICE_NOT_AVAILABLE        0x3F
#define UC_CAP_NOT_IMPLEMENTED          0x41
#define UC_CHAN_NOT_IMPLEMENTED         0x42
#define UC_FACILITY_NOT_IMPLEMENTED     0x45
#define UC_INVALID_CALL_REF             0x51
#define UC_CHAN_DOES_NOT_EXIST          0x52
#define UC_INCOMPATIBLE_DEST            0x58
#define UC_INVALID_MSG_UNSPECIFIED      0x5F
#define UC_MANDATORY_IE_MISSING         0x60
#define UC_NONEXISTENT_MSG              0x61
#define UC_WRONG_MESSAGE                0x62
#define UC_BAD_INFO_ELEM                0x63
#define UC_INVALID_ELEM_CONTENTS        0x64
#define UC_WRONG_MSG_FOR_STATE          0x65
#define UC_TIMER_EXPIRY                 0x66
#define UC_MANDATORY_IE_LEN_ERROR       0x67
#define UC_PROTOCOL_ERROR               0x6F
#define UC_INTERWORKING_UNSPECIFIED     0x7F

#define MFCR2_DEBUG_TONE_ON             0x0001
#define MFCR2_DEBUG_TONE_OFF            0x0002
#define MFCR2_DEBUG_TONE_SEND           0x0004
#define MFCR2_DEBUG_CAS                 0x0008
#define MFCR2_DEBUG_FLOW                0x0010

typedef struct r2_state_s r2_state_t;
typedef struct mfcr2_s mfcr2_t;
typedef struct mfcr2_call_s mfcr2_call_t;
typedef struct mfcr2_makecall_s mfcr2_makecall_t;

enum
{
    MFCR2_PROT_ARGENTINA = 1,
    MFCR2_PROT_BAHRAIN,
    MFCR2_PROT_BOLIVIA,
    MFCR2_PROT_BRAZIL,
    MFCR2_PROT_CHILE,
    MFCR2_PROT_CHINA,
    MFCR2_PROT_COLOMBIA_LAND,
    MFCR2_PROT_COLOMBIA_CELL,
    MFCR2_PROT_CZECH,
    MFCR2_PROT_HONDURAS,
    MFCR2_PROT_INDIA,
    MFCR2_PROT_INDONESIA,
    MFCR2_PROT_KOREA,
    MFCR2_PROT_MALAYSIA,
    MFCR2_PROT_MEXICO,
    MFCR2_PROT_PANAMA,
    MFCR2_PROT_PHILIPINNES,
    MFCR2_PROT_SINGAPORE,
    MFCR2_PROT_THAILAND
};

enum
{
    MFCR2_EVENT_BLOCKED = 1,
    MFCR2_EVENT_UNBLOCKED,
    MFCR2_EVENT_CONFIG_ERR,
    MFCR2_EVENT_RING,
    MFCR2_EVENT_HANGUP,
    MFCR2_EVENT_RINGING,
    MFCR2_EVENT_ANSWER,
    MFCR2_EVENT_HANGUP_ACK,
    MFCR2_EVENT_RESTART_ACK,
    MFCR2_EVENT_IDLE
};

typedef struct mfcr2_event_generic_s
{
    /* Events with no additional information fall in this category */
    int e;
} mfcr2_event_generic_t;

typedef struct mfcr2_event_error_s
{
    int e;
    char err[256];
} mfcr2_event_error_t;

typedef struct mfcr2_event_restart_s
{
    int e;
    int channel;
} mfcr2_event_restart_t;

typedef struct mfcr2_event_answer_s
{
    int e;
    int channel;
    int cref;
} mfcr2_event_answer_t;

typedef struct mfcr2_event_ring_s
{
    int e;
    int channel;		/* Channel requested */
    int callingpres;		/* Presentation of Calling CallerID */
    int callingplan;		/* Dialing plan of Calling entity */
    char callingnum[256];	/* Calling number */
    uint8_t redirecting_pres;	/* Presentation of redirecting number */
    uint8_t redirecting_ton;
    uint8_t redirecting_npi;
    int redirecting_cause;	/* Reason for redirection */
    char redirectingnum[256];	/* Redirecting number */
    int calledplan;		/* Dialing plan of Called number */
    char callednum[256];	/* Called number */
    int flexible;		/* Are we flexible with our channel selection? */
    int cref;			/* Call Reference Number */
    int ctype;			/* Call type (see mfcr2_TRANS_CAP_* */
    int layer1;			/* User layer 1 */
} mfcr2_event_ring_t;

typedef struct mfcr2_event_hangup_s
{
    int e;
    int channel;		/* Channel requested */
    int cause;
    int cref;
} mfcr2_event_hangup_t;	

typedef struct mfcr2_event_restart_ack_s
{
    int e;
    int channel;
} mfcr2_event_restart_ack_t;

typedef union
{
    int e;
    mfcr2_event_generic_t gen;		    /* Generic view */
    mfcr2_event_restart_t restart;	    /* Restart view */
    mfcr2_event_error_t	  err;		    /* Error view */
    mfcr2_event_ring_t	  ring;	            /* Ring */
    mfcr2_event_answer_t  answer;	    /* Connected */
    mfcr2_event_hangup_t  hangup;	    /* Hang up */
    mfcr2_event_restart_ack_t restartack;   /* Restart Acknowledge */
} mfcr2_event_t;

extern int mfcr2_MakeCall (mfcr2_t *mfcr2, mfcr2_call_t *call, mfcr2_makecall_t *makecall);
extern int mfcr2_AnswerCall (mfcr2_t *mfcr2, mfcr2_call_t *call);
extern int mfcr2_DropCall (mfcr2_t *mfcr2, mfcr2_call_t *call, int cause);
extern int mfcr2_Unblock (mfcr2_t *mfcr2);
extern int mfcr2_Block (mfcr2_t *mfcr2);

extern mfcr2_t *mfcr2_new (int fd, int protocol, int outgoing_calls_allowed);
extern void mfcr2_free (mfcr2_t *mfcr2);

extern mfcr2_call_t *mfcr2_new_call (mfcr2_t *mfcr2);
extern mfcr2_makecall_t *mfcr2_new_makecall(void);
extern void mfcr2_callparm_originating_number(mfcr2_makecall_t *parms, char *num);
extern void mfcr2_callparm_destination_number(mfcr2_makecall_t *parms, char *num);

extern void mfcr2_set_debug(mfcr2_t *mfcr2, int level);
extern char *mfcr2_event2str(int id);
extern mfcr2_event_t *mfcr2_check_event(mfcr2_t *mfcr2);
extern struct timeval *mfcr2_schedule_next(mfcr2_t *mfcr2);
extern mfcr2_event_t *mfcr2_schedule_run (mfcr2_t *mfcr2);
extern mfcr2_event_t *mfcr2_cas_signaling_event (mfcr2_t *mfcr2, unsigned int sig);


#endif
